@@ -1,5 +1,5 @@
 /* AIX FPU-related code.
-   Copyright (C) 2005-2020 Free Software Foundation, Inc.
+   Copyright (C) 2005-2025 Free Software Foundation, Inc.
    Contributed by Francois-Xavier Coudert <coudert@clipper.ens.fr>
 
 This file is part of the GNU Fortran runtime library (libgfortran).
@@ -320,6 +320,11 @@ get_fpu_rounding_mode (void)
 	return GFC_FPE_TOWARDZERO;
 #endif
 
+#ifdef FE_TONEARESTFROMZERO
+      case FE_TONEARESTFROMZERO:
+	return GFC_FPE_AWAY;
+#endif
+
       default:
 	return 0; /* Should be unreachable.  */
     }
@@ -357,8 +362,14 @@ set_fpu_rounding_mode (int mode)
 	break;
 #endif
 
+#ifdef FE_TONEARESTFROMZERO
+      case GFC_FPE_AWAY:
+	rnd_mode = FE_TONEARESTFROMZERO;
+	break;
+#endif
+
       default:
-	return; /* Should be unreachable.  */
+	return;
     }
 
   fesetround (rnd_mode);
@@ -398,8 +409,15 @@ support_fpu_rounding_mode (int mode)
 	return 0;
 #endif
 
+      case GFC_FPE_AWAY:
+#ifdef FE_TONEARESTFROMZERO
+	return 1;
+#else
+	return 0;
+#endif
+
       default:
-	return 0; /* Should be unreachable.  */
+	return 0;
     }
 }
 
